package it.torvergata.example.solrproject;

import java.io.IOException;

import org.apache.solr.client.solrj.SolrQuery;
import org.apache.solr.client.solrj.SolrQuery.ORDER;
import org.apache.solr.client.solrj.SolrRequest;
import org.apache.solr.client.solrj.SolrServerException;
import org.apache.solr.client.solrj.impl.HttpSolrClient;
import org.apache.solr.client.solrj.impl.XMLResponseParser;
import org.apache.solr.client.solrj.response.QueryResponse;
import org.apache.solr.common.SolrDocumentList;
import org.apache.solr.common.SolrInputDocument;
import org.springframework.stereotype.Component;

@Component
public class Executor {
    public static final String SCORE = "score";
    public static final String STAR_SCORE = "* score";
    public static final String DEF_TYPE_KEY = "defType";
    public static final String DIS_MAX_KEY = "edismax";

    public HttpSolrClient getConnecton() {
        String urlString = "http://localhost:8983/solr/mioesempio";
        HttpSolrClient solr = new HttpSolrClient.Builder(urlString).build();
        solr.setParser(new XMLResponseParser());
        System.out.println("Get connection");
        return solr;

    }

    public void index() throws SolrServerException, IOException {

        HttpSolrClient solrClient = getConnecton();
        SolrInputDocument document = new SolrInputDocument();
        document.addField("id", "987654");
        document.addField("name", "Acer Nitro 5");
        document.addField("price", "599.99");
        solrClient.add(document);
        solrClient.commit();

    }

    public void indexBean() throws SolrServerException, IOException {

        HttpSolrClient solrClient = getConnecton();

        solrClient.setParser(new XMLResponseParser());
        solrClient.addBean(new Product("888", "Apple iPhone 20", "111299.99"));
        solrClient.commit();
    }

    public void query() throws SolrServerException, IOException {
        HttpSolrClient solrClient = getConnecton();

        SolrQuery queryParams = new SolrQuery();
        queryParams.setQuery("Apple");
        queryParams.setSort(SCORE, ORDER.desc);
        queryParams.set("fl", STAR_SCORE);
        queryParams.set("qf", "name^2");
        queryParams.set(DEF_TYPE_KEY, DIS_MAX_KEY);
        queryParams.setRows(100)
            .setStart(0);

        final QueryResponse response = solrClient.query(queryParams, SolrRequest.METHOD.POST);
        final SolrDocumentList results = response.getResults();
        System.out.println(results);
    }

    public void query2() throws SolrServerException, IOException {
        HttpSolrClient solrClient = getConnecton();

        SolrQuery queryParams = new SolrQuery();
        queryParams.setQuery("name:*");
        queryParams.set("fl", STAR_SCORE);
        queryParams.set("fq", "price:[0 TO 399}");
        queryParams.set("sort", "price desc");

        queryParams.set(DEF_TYPE_KEY, DIS_MAX_KEY);
        queryParams.setRows(100)
            .setStart(0);

        final QueryResponse response = solrClient.query(queryParams, SolrRequest.METHOD.POST);
        final SolrDocumentList results = response.getResults();
        System.out.println(results);
    }

    public void query3() throws SolrServerException, IOException {
        HttpSolrClient solrClient = getConnecton();

        SolrQuery queryParams = new SolrQuery();
        queryParams.setQuery("name:\"iPod Cable\"~1");
        queryParams.set("fl", STAR_SCORE);
        queryParams.set("sort", "price desc");

        queryParams.set(DEF_TYPE_KEY, DIS_MAX_KEY);
        queryParams.setRows(100)
            .setStart(0);

        final QueryResponse response = solrClient.query(queryParams, SolrRequest.METHOD.POST);
        final SolrDocumentList results = response.getResults();
        System.out.println(results);
    }

    public void query4() throws SolrServerException, IOException {
        HttpSolrClient solrClient = getConnecton();

        SolrQuery queryParams = new SolrQuery();
        queryParams.setQuery("*:*");
        queryParams.set("fl", STAR_SCORE);
        queryParams.set("fq", "cat:\"graphics card\"");
        queryParams.set("sort", "price asc");

        queryParams.set(DEF_TYPE_KEY, DIS_MAX_KEY);
        queryParams.setRows(100)
            .setStart(0);

        final QueryResponse response = solrClient.query(queryParams, SolrRequest.METHOD.POST);
        final SolrDocumentList results = response.getResults();
        System.out.println(results);
    }

    public void query5() throws SolrServerException, IOException {
        HttpSolrClient solrClient = getConnecton();

        SolrQuery queryParams = new SolrQuery();
        queryParams.setQuery("*:*");
        queryParams.set("fl", STAR_SCORE);
        queryParams.set("fq", "cat:\"graphics card\" AND price:[0 TO 500}");
        queryParams.set("sort", "price asc");

        queryParams.set(DEF_TYPE_KEY, DIS_MAX_KEY);
        queryParams.setRows(100)
            .setStart(0);

        final QueryResponse response = solrClient.query(queryParams, SolrRequest.METHOD.POST);
        final SolrDocumentList results = response.getResults();
        System.out.println(results);
    }

    public void query6() throws SolrServerException, IOException {
        HttpSolrClient solrClient = getConnecton();

        SolrQuery queryParams = new SolrQuery();
        queryParams.setQuery("(name:Canon OR name:ATI ) AND (features:scanner OR features:svideo)");
        queryParams.setSort(SCORE, ORDER.desc);
        queryParams.set("fl", STAR_SCORE);
        queryParams.set("fq", "price:[0 TO 1000}");

        queryParams.set(DEF_TYPE_KEY, DIS_MAX_KEY);
        queryParams.setRows(100)
            .setStart(0);

        final QueryResponse response = solrClient.query(queryParams, SolrRequest.METHOD.POST);
        final SolrDocumentList results = response.getResults();
        System.out.println(results);
    }
}
